@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -126,8 +126,12 @@ class GlancesAttribute(object):
         """
         return self._history[-pos]
 
+    def history_raw(self, nb=0):
+        """Return the history in ISO JSON format"""
+        return self._history[-nb:]
+
     def history_json(self, nb=0):
-        """Return the history of last nb items (0 for all) In ISO JSON format"""
+        """Return the history in ISO JSON format"""
         return [(i[0].isoformat(), i[1]) for i in self._history[-nb:]]
 
     def history_mean(self, nb=5):
