@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -25,24 +25,16 @@ import operator
 import sys
 import unicodedata
 import types
-import platform
+import subprocess
+
+from glances.logger import logger
 
-PY_CYTHON = platform.python_implementation() == 'CPython'
-PY_PYPY = platform.python_implementation() == 'PyPy'
-PY_JYTHON = platform.python_implementation() == 'Jython'
-PY_IRON = platform.python_implementation() == 'IronPython'
 PY3 = sys.version_info[0] == 3
 
-try:
-    from statistics import mean
-except ImportError:
-    # Statistics is only available for Python 3.4 or higher
-    def mean(numbers):
-        return float(sum(numbers)) / max(len(numbers), 1)
-
 if PY3:
     import queue
     from configparser import ConfigParser, NoOptionError, NoSectionError
+    from statistics import mean
     from xmlrpc.client import Fault, ProtocolError, ServerProxy, Transport, Server
     from xmlrpc.server import SimpleXMLRPCRequestHandler, SimpleXMLRPCServer
     from urllib.request import urlopen
@@ -56,6 +48,7 @@ if PY3:
     text_type = str
     binary_type = bytes
     bool_type = bool
+    long = int
 
     viewkeys = operator.methodcaller('keys')
     viewvalues = operator.methodcaller('values')
@@ -64,7 +57,9 @@ if PY3:
     def to_ascii(s):
         """Convert the bytes string to a ASCII string
         Usefull to remove accent (diacritics)"""
-        return str(s, 'utf-8')
+        if isinstance(s, binary_type):
+            return s.decode()
+        return s.encode('ascii', 'ignore').decode()
 
     def listitems(d):
         return list(d.items())
@@ -92,12 +87,23 @@ if PY3:
     def b(s):
         if isinstance(s, binary_type):
             return s
-        return s.encode('latin-1')
+        return s.encode('utf-8')
 
     def nativestr(s):
         if isinstance(s, text_type):
             return s
         return s.decode('utf-8', 'replace')
+
+    def system_exec(command):
+        """Execute a system command and return the resul as a str"""
+        try:
+            res = subprocess.run(command.split(' '),
+                                 stdout=subprocess.PIPE).stdout.decode('utf-8')
+        except Exception as e:
+            logger.debug('Can not evaluate command {} ({})'.format(command, e))
+            res = ''
+        return res.rstrip()
+
 else:
     import Queue as queue
     from itertools import imap as map
@@ -114,17 +120,21 @@ else:
     text_type = unicode
     binary_type = str
     bool_type = types.BooleanType
+    long = long
 
     viewkeys = operator.methodcaller('viewkeys')
     viewvalues = operator.methodcaller('viewvalues')
     viewitems = operator.methodcaller('viewitems')
 
+    def mean(numbers):
+        return float(sum(numbers)) / max(len(numbers), 1)
+
     def to_ascii(s):
         """Convert the unicode 's' to a ASCII string
         Usefull to remove accent (diacritics)"""
         if isinstance(s, binary_type):
             return s
-        return unicodedata.normalize('NFKD', s).encode('ASCII', 'ignore')
+        return unicodedata.normalize('NFKD', s).encode('ascii', 'ignore')
 
     def listitems(d):
         return d.items()
@@ -158,3 +168,45 @@ else:
         if isinstance(s, binary_type):
             return s
         return s.encode('utf-8', 'replace')
+
+    def system_exec(command):
+        """Execute a system command and return the resul as a str"""
+        try:
+            res = subprocess.check_output(command.split(' '))
+        except Exception as e:
+            logger.debug('Can not execute command {} ({})'.format(command, e))
+            res = ''
+        return res.rstrip()
+
+
+# Globals functions for both Python 2 and 3
+
+
+def subsample(data, sampling):
+    """Compute a simple mean subsampling.
+
+    Data should be a list of numerical itervalues
+
+    Return a subsampled list of sampling lenght
+    """
+    if len(data) <= sampling:
+        return data
+    sampling_length = int(round(len(data) / float(sampling)))
+    return [mean(data[s * sampling_length:(s + 1) * sampling_length]) for s in range(0, sampling)]
+
+
+def time_serie_subsample(data, sampling):
+    """Compute a simple mean subsampling.
+
+    Data should be a list of set (time, value)
+
+    Return a subsampled list of sampling lenght
+    """
+    if len(data) <= sampling:
+        return data
+    t = [t[0] for t in data]
+    v = [t[1] for t in data]
+    sampling_length = int(round(len(data) / float(sampling)))
+    t_subsampled = [t[s * sampling_length:(s + 1) * sampling_length][0] for s in range(0, sampling)]
+    v_subsampled = [mean(v[s * sampling_length:(s + 1) * sampling_length]) for s in range(0, sampling)]
+    return list(zip(t_subsampled, v_subsampled))
