@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2015 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -21,7 +21,7 @@
 
 import os
 
-from glances.compat import range
+from glances.compat import range, nativestr
 from glances.logger import logger
 
 # Use the built-in version of scandir/walk if possible, otherwise
@@ -81,20 +81,16 @@ class FolderList(object):
             key = 'folder_' + str(l) + '_'
 
             # Path is mandatory
-            try:
-                value['path'] = self.config.get_value(section, key + 'path')
-            except Exception as e:
-                logger.error("Cannot read folder list: {}".format(e))
-                continue
+            value['path'] = self.config.get_value(section, key + 'path')
             if value['path'] is None:
                 continue
+            else:
+                value['path'] = nativestr(value['path'])
 
             # Optional conf keys
             for i in ['careful', 'warning', 'critical']:
-                try:
-                    value[i] = self.config.get_value(section, key + i)
-                except:
-                    value[i] = None
+                value[i] = self.config.get_value(section, key + i)
+                if value[i] is None:
                     logger.debug("No {} threshold for folder {}".format(i, value["path"]))
 
             # Add the item to the list
@@ -153,7 +149,7 @@ class FolderList(object):
             # Update folder size
             try:
                 self.__folder_list[i]['size'] = self.__folder_size(self.path(i))
-            except Exception as e:
+            except OSError as e:
                 logger.debug('Cannot get folder size ({}). Error: {}'.format(self.path(i), e))
                 if e.errno == 13:
                     # Permission denied
