@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -95,5 +95,5 @@ class Bar(object):
             whole += 1
         ret += self.__empty_char * int(self.size - whole)
         if self.__with_text:
-            ret = '{}{:>5}%'.format(ret, self.percent)
+            ret = '{}{:5.1f}%'.format(ret, self.percent)
         return ret
