@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Kirby Banman <kirby.banman@gmail.com>
+# Copyright (C) 2018 Kirby Banman <kirby.banman@gmail.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -17,30 +17,32 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-"""GPU plugin (limited to NVIDIA chipsets)"""
+"""GPU plugin (limited to NVIDIA chipsets)."""
 
+from glances.compat import nativestr
 from glances.logger import logger
 from glances.plugins.glances_plugin import GlancesPlugin
 
 try:
     import pynvml
-except ImportError:
-    logger.debug("Could not import pynvml.  NVIDIA stats will not be collected.")
-    gpu_nvidia_tag = False
+except Exception as e:
+    import_error_tag = True
+    # Display debu message if import KeyError
+    logger.warning("Missing Python Lib ({}), Nvidia GPU plugin is disabled".format(e))
 else:
-    gpu_nvidia_tag = True
+    import_error_tag = False
 
 
 class Plugin(GlancesPlugin):
-
     """Glances GPU plugin (limited to NVIDIA chipsets).
 
     stats is a list of dictionaries with one entry per GPU
     """
 
     def __init__(self, args=None):
-        """Init the plugin"""
-        super(Plugin, self).__init__(args=args)
+        """Init the plugin."""
+        super(Plugin, self).__init__(args=args,
+                                     stats_init_value=[])
 
         # Init the NVidia API
         self.init_nvidia()
@@ -48,16 +50,9 @@ class Plugin(GlancesPlugin):
         # We want to display the stat in the curse interface
         self.display_curse = True
 
-        # Init the stats
-        self.reset()
-
-    def reset(self):
-        """Reset/init the stats."""
-        self.stats = []
-
     def init_nvidia(self):
-        """Init the NVIDIA API"""
-        if not gpu_nvidia_tag:
+        """Init the NVIDIA API."""
+        if import_error_tag:
             self.nvml_ready = False
 
         try:
@@ -77,11 +72,11 @@ class Plugin(GlancesPlugin):
     @GlancesPlugin._check_decorator
     @GlancesPlugin._log_result_decorator
     def update(self):
-        """Update the GPU stats"""
+        """Update the GPU stats."""
+        # Init new stats
+        stats = self.get_init_value()
 
-        self.reset()
-
-        # !!! JUST FOR TEST
+        # !!! JUST FOR TEST (because i did not have any NVidia GPU... :()
         # self.stats = [{"key": "gpu_id", "mem": None, "proc": 60, "gpu_id": 0, "name": "GeForce GTX 560 Ti"}]
         # self.stats = [{"key": "gpu_id", "mem": 10, "proc": 60, "gpu_id": 0, "name": "GeForce GTX 560 Ti"}]
         # self.stats = [{"key": "gpu_id", "mem": 48.64645, "proc": 60.73, "gpu_id": 0, "name": "GeForce GTX 560 Ti"},
@@ -96,11 +91,14 @@ class Plugin(GlancesPlugin):
             return self.stats
 
         if self.input_method == 'local':
-            self.stats = self.get_device_stats()
+            stats = self.get_device_stats()
         elif self.input_method == 'snmp':
             # not available
             pass
 
+        # Update the stats
+        self.stats = stats
+
         return self.stats
 
     def update_views(self):
@@ -210,7 +208,7 @@ class Plugin(GlancesPlugin):
         return ret
 
     def get_device_stats(self):
-        """Get GPU stats"""
+        """Get GPU stats."""
         stats = []
 
         for index, device_handle in enumerate(self.device_handles):
@@ -230,7 +228,7 @@ class Plugin(GlancesPlugin):
         return stats
 
     def exit(self):
-        """Overwrite the exit method to close the GPU API"""
+        """Overwrite the exit method to close the GPU API."""
         if self.nvml_ready:
             try:
                 pynvml.nvmlShutdown()
@@ -242,22 +240,23 @@ class Plugin(GlancesPlugin):
 
 
 def get_device_handles():
-    """
-    Returns a list of NVML device handles, one per device.  Can throw NVMLError.
+    """Get a list of NVML device handles, one per device.
+
+    Can throw NVMLError.
     """
     return [pynvml.nvmlDeviceGetHandleByIndex(i) for i in range(pynvml.nvmlDeviceGetCount())]
 
 
 def get_device_name(device_handle):
-    """Get GPU device name"""
+    """Get GPU device name."""
     try:
-        return pynvml.nvmlDeviceGetName(device_handle)
+        return nativestr(pynvml.nvmlDeviceGetName(device_handle))
     except pynvml.NVMlError:
         return "NVIDIA"
 
 
 def get_mem(device_handle):
-    """Get GPU device memory consumption in percent"""
+    """Get GPU device memory consumption in percent."""
     try:
         memory_info = pynvml.nvmlDeviceGetMemoryInfo(device_handle)
         return memory_info.used * 100.0 / memory_info.total
@@ -266,7 +265,7 @@ def get_mem(device_handle):
 
 
 def get_proc(device_handle):
-    """Get GPU device CPU consumption in percent"""
+    """Get GPU device CPU consumption in percent."""
     try:
         return pynvml.nvmlDeviceGetUtilizationRates(device_handle).gpu
     except pynvml.NVMLError:
