@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2018 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,11 +20,12 @@
 """Process list plugin."""
 
 import os
+import shlex
+import copy
 from datetime import timedelta
 
-from glances.compat import iteritems
-from glances.globals import LINUX, WINDOWS
 from glances.logger import logger
+from glances.globals import WINDOWS
 from glances.processes import glances_processes, sort_stats
 from glances.plugins.glances_core import Plugin as CorePlugin
 from glances.plugins.glances_plugin import GlancesPlugin
@@ -44,28 +45,55 @@ def convert_timedelta(delta):
 def split_cmdline(cmdline):
     """Return path, cmd and arguments for a process cmdline."""
     path, cmd = os.path.split(cmdline[0])
-    arguments = ' '.join(cmdline[1:]).replace('\n', ' ')
-    # XXX: workaround for psutil issue #742
-    if LINUX and any(x in cmdline[0] for x in ('chrome', 'chromium')):
-        try:
-            exe, arguments = cmdline[0].split(' ', 1)
-            path, cmd = os.path.split(exe)
-        except ValueError:
-            arguments = None
-
+    arguments = ' '.join(cmdline[1:])
     return path, cmd, arguments
 
 
 class Plugin(GlancesPlugin):
-
     """Glances' processes plugin.
 
     stats is a list
     """
 
+    # Define the header layout of the processes list columns
+    layout_header = {
+        'cpu': '{:<6} ',
+        'mem': '{:<5} ',
+        'virt': '{:<5} ',
+        'res': '{:<5} ',
+        'pid': '{:>{width}} ',
+        'user': '{:<10} ',
+        'time': '{:>8} ',
+        'thread': '{:<3} ',
+        'nice': '{:>3} ',
+        'status': '{:>1} ',
+        'ior': '{:>4} ',
+        'iow': '{:<4} ',
+        'command': '{}',
+    }
+
+    # Define the stat layout of the processes list columns
+    layout_stat = {
+        'cpu': '{:<6.1f} ',
+        'mem': '{:<5.1f} ',
+        'virt': '{:<5} ',
+        'res': '{:<5} ',
+        'pid': '{:>{width}} ',
+        'user': '{:<10} ',
+        'time': '{:>8} ',
+        'thread': '{:<3} ',
+        'nice': '{:>3} ',
+        'status': '{:>1} ',
+        'ior': '{:>4} ',
+        'iow': '{:<4} ',
+        'command': '{}',
+        'name': '[{}]'
+    }
+
     def __init__(self, args=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args)
+        super(Plugin, self).__init__(args=args,
+                                     stats_init_value=[])
 
         # We want to display the stat in the curse interface
         self.display_curse = True
@@ -80,7 +108,7 @@ class Plugin(GlancesPlugin):
             self.nb_log_core = 0
 
         # Get the max values (dict)
-        self.max_values = glances_processes.max_values()
+        self.max_values = copy.deepcopy(glances_processes.max_values())
 
         # Get the maximum PID number
         # Use to optimize space (see https://github.com/nicolargo/glances/issues/959)
@@ -92,116 +120,53 @@ class Plugin(GlancesPlugin):
         """Return the key of the list."""
         return 'pid'
 
-    def reset(self):
-        """Reset/init the stats."""
-        self.stats = []
-
     def update(self):
         """Update processes stats using the input method."""
-        # Reset stats
-        self.reset()
+        # Init new stats
+        stats = self.get_init_value()
 
         if self.input_method == 'local':
             # Update stats using the standard system lib
             # Note: Update is done in the processcount plugin
             # Just return the processes list
-            if glances_processes.is_tree_enabled():
-                self.stats = glances_processes.gettree()
-            else:
-                self.stats = glances_processes.getlist()
-
-            # Get the max values (dict)
-            self.max_values = glances_processes.max_values()
+            stats = glances_processes.getlist()
 
         elif self.input_method == 'snmp':
             # No SNMP grab for processes
             pass
 
+        # Update the stats
+        self.stats = stats
+
+        # Get the max values (dict)
+        # Use Deep copy to avoid change between update and display
+        self.max_values = copy.deepcopy(glances_processes.max_values())
+
         return self.stats
 
-    def get_process_tree_curses_data(self, node, args, first_level=True, max_node_count=None):
-        """Get curses data to display for a process tree."""
-        ret = []
-        node_count = 0
-        if not node.is_root and ((max_node_count is None) or (max_node_count > 0)):
-            node_data = self.get_process_curses_data(node.stats, False, args)
-            node_count += 1
-            ret.extend(node_data)
-        for child in node.iter_children():
-            # stop if we have enough nodes to display
-            if max_node_count is not None and node_count >= max_node_count:
-                break
-
-            if max_node_count is None:
-                children_max_node_count = None
-            else:
-                children_max_node_count = max_node_count - node_count
-            child_data = self.get_process_tree_curses_data(child,
-                                                           args,
-                                                           first_level=node.is_root,
-                                                           max_node_count=children_max_node_count)
-            if max_node_count is None:
-                node_count += len(child)
-            else:
-                node_count += min(children_max_node_count, len(child))
-
-            if not node.is_root:
-                child_data = self.add_tree_decoration(child_data, child is node.children[-1], first_level)
-            ret.extend(child_data)
-        return ret
-
-    def add_tree_decoration(self, child_data, is_last_child, first_level):
-        """Add tree curses decoration and indentation to a subtree."""
-        # find process command indices in messages
-        pos = []
-        for i, m in enumerate(child_data):
-            if m.get("_tree_decoration", False):
-                del m["_tree_decoration"]
-                pos.append(i)
-
-        # add new curses items for tree decoration
-        new_child_data = []
-        new_pos = []
-        for i, m in enumerate(child_data):
-            if i in pos:
-                new_pos.append(len(new_child_data))
-                new_child_data.append(self.curse_add_line(""))
-                new_child_data[-1]["_tree_decoration"] = True
-            new_child_data.append(m)
-        child_data = new_child_data
-        pos = new_pos
-
-        if pos:
-            # draw node prefix
-            if is_last_child:
-                prefix = "└─"
-            else:
-                prefix = "├─"
-            child_data[pos[0]]["msg"] = prefix
-
-            # add indentation
-            for i in pos:
-                spacing = 2
-                if first_level:
-                    spacing = 1
-                elif is_last_child and (i is not pos[0]):
-                    # compensate indentation for missing '│' char
-                    spacing = 3
-                child_data[i]["msg"] = "%s%s" % (" " * spacing, child_data[i]["msg"])
-
-            if not is_last_child:
-                # add '│' tree decoration
-                for i in pos[1:]:
-                    old_str = child_data[i]["msg"]
-                    if first_level:
-                        child_data[i]["msg"] = " │" + old_str[2:]
-                    else:
-                        child_data[i]["msg"] = old_str[:2] + "│" + old_str[3:]
-
-        return child_data
+    def get_nice_alert(self, value):
+        """Return the alert relative to the Nice configuration list"""
+        value = str(value)
+        try:
+            if value in self.get_limit('nice_critical'):
+                return 'CRITICAL'
+        except KeyError:
+            pass
+        try:
+            if value in self.get_limit('nice_warning'):
+                return 'WARNING'
+        except KeyError:
+            pass
+        try:
+            if value in self.get_limit('nice_careful'):
+                return 'CAREFUL'
+        except KeyError:
+            pass
+        return 'DEFAULT'
 
     def get_process_curses_data(self, p, first, args):
         """Get curses data to display for a process.
+
         - p is the process to display
         - first is a tag=True if the process is the first on the list
         """
@@ -209,154 +174,164 @@ class Plugin(GlancesPlugin):
         # CPU
         if 'cpu_percent' in p and p['cpu_percent'] is not None and p['cpu_percent'] != '':
             if args.disable_irix and self.nb_log_core != 0:
-                msg = '{:>6.1f}'.format(p['cpu_percent'] / float(self.nb_log_core))
+                msg = self.layout_stat['cpu'].format(p['cpu_percent'] / float(self.nb_log_core))
             else:
-                msg = '{:>6.1f}'.format(p['cpu_percent'])
+                msg = self.layout_stat['cpu'].format(p['cpu_percent'])
             alert = self.get_alert(p['cpu_percent'],
                                    highlight_zero=False,
                                    is_max=(p['cpu_percent'] == self.max_values['cpu_percent']),
                                    header="cpu")
             ret.append(self.curse_add_line(msg, alert))
         else:
-            msg = '{:>6}'.format('?')
+            msg = self.layout_header['cpu'].format('?')
             ret.append(self.curse_add_line(msg))
         # MEM
         if 'memory_percent' in p and p['memory_percent'] is not None and p['memory_percent'] != '':
-            msg = '{:>6.1f}'.format(p['memory_percent'])
+            msg = self.layout_stat['mem'].format(p['memory_percent'])
             alert = self.get_alert(p['memory_percent'],
                                    highlight_zero=False,
                                    is_max=(p['memory_percent'] == self.max_values['memory_percent']),
                                    header="mem")
             ret.append(self.curse_add_line(msg, alert))
         else:
-            msg = '{:>6}'.format('?')
+            msg = self.layout_header['mem'].format('?')
             ret.append(self.curse_add_line(msg))
         # VMS/RSS
         if 'memory_info' in p and p['memory_info'] is not None and p['memory_info'] != '':
             # VMS
-            msg = '{:>6}'.format(self.auto_unit(p['memory_info'][1], low_precision=False))
+            msg = self.layout_stat['virt'].format(self.auto_unit(p['memory_info'][1], low_precision=False))
             ret.append(self.curse_add_line(msg, optional=True))
             # RSS
-            msg = '{:>6}'.format(self.auto_unit(p['memory_info'][0], low_precision=False))
+            msg = self.layout_stat['res'].format(self.auto_unit(p['memory_info'][0], low_precision=False))
             ret.append(self.curse_add_line(msg, optional=True))
         else:
-            msg = '{:>6}'.format('?')
+            msg = self.layout_header['virt'].format('?')
             ret.append(self.curse_add_line(msg))
+            msg = self.layout_header['res'].format('?')
             ret.append(self.curse_add_line(msg))
         # PID
-        msg = '{:>{width}}'.format(p['pid'], width=self.__max_pid_size() + 1)
+        msg = self.layout_stat['pid'].format(p['pid'], width=self.__max_pid_size())
         ret.append(self.curse_add_line(msg))
         # USER
         if 'username' in p:
             # docker internal users are displayed as ints only, therefore str()
             # Correct issue #886 on Windows OS
-            msg = ' {:9}'.format(str(p['username'])[:9])
+            msg = self.layout_stat['user'].format(str(p['username'])[:9])
             ret.append(self.curse_add_line(msg))
         else:
-            msg = ' {:9}'.format('?')
+            msg = self.layout_header['user'].format('?')
+            ret.append(self.curse_add_line(msg))
+        # TIME+
+        try:
+            delta = timedelta(seconds=sum(p['cpu_times']))
+        except (OverflowError, TypeError) as e:
+            # Catch OverflowError on some Amazon EC2 server
+            # See https://github.com/nicolargo/glances/issues/87
+            # Also catch TypeError on macOS
+            # See: https://github.com/nicolargo/glances/issues/622
+            # logger.debug("Cannot get TIME+ ({})".format(e))
+            msg = self.layout_header['time'].format('?')
+            ret.append(self.curse_add_line(msg, optional=True))
+        else:
+            hours, minutes, seconds, microseconds = convert_timedelta(delta)
+            if hours > 99:
+                msg = '{:<7}h'.format(hours)
+            elif 0 < hours < 100:
+                msg = '{}h{}:{}'.format(hours, minutes, seconds)
+            else:
+                msg = '{}:{}'.format(minutes, seconds)
+            msg = self.layout_stat['time'].format(msg)
+            if hours > 0:
+                ret.append(self.curse_add_line(msg,
+                                               decoration='CPU_TIME',
+                                               optional=True))
+            else:
+                ret.append(self.curse_add_line(msg, optional=True))
+        # THREAD
+        if 'num_threads' in p:
+            num_threads = p['num_threads']
+            if num_threads is None:
+                num_threads = '?'
+            msg = self.layout_stat['thread'].format(num_threads)
+            ret.append(self.curse_add_line(msg))
+        else:
+            msg = self.layout_header['thread'].format('?')
             ret.append(self.curse_add_line(msg))
         # NICE
         if 'nice' in p:
             nice = p['nice']
             if nice is None:
                 nice = '?'
-            msg = '{:>5}'.format(nice)
-            if isinstance(nice, int) and ((WINDOWS and nice != 32) or
-                                          (not WINDOWS and nice != 0)):
-                ret.append(self.curse_add_line(msg, decoration='NICE'))
-            else:
-                ret.append(self.curse_add_line(msg))
+            msg = self.layout_stat['nice'].format(nice)
+            ret.append(self.curse_add_line(msg,
+                                           decoration=self.get_nice_alert(nice)))
         else:
-            msg = '{:>5}'.format('?')
+            msg = self.layout_header['nice'].format('?')
             ret.append(self.curse_add_line(msg))
         # STATUS
         if 'status' in p:
             status = p['status']
-            msg = '{:>2}'.format(status)
+            msg = self.layout_stat['status'].format(status)
             if status == 'R':
                 ret.append(self.curse_add_line(msg, decoration='STATUS'))
             else:
                 ret.append(self.curse_add_line(msg))
         else:
-            msg = '{:>2}'.format('?')
+            msg = self.layout_header['status'].format('?')
             ret.append(self.curse_add_line(msg))
-        # TIME+
-        if self.tag_proc_time:
-            try:
-                delta = timedelta(seconds=sum(p['cpu_times']))
-            except (OverflowError, TypeError) as e:
-                # Catch OverflowError on some Amazon EC2 server
-                # See https://github.com/nicolargo/glances/issues/87
-                # Also catch TypeError on macOS
-                # See: https://github.com/nicolargo/glances/issues/622
-                logger.debug("Cannot get TIME+ ({})".format(e))
-                self.tag_proc_time = False
-            else:
-                hours, minutes, seconds, microseconds = convert_timedelta(delta)
-                if hours:
-                    msg = '{:>4}h'.format(hours)
-                    ret.append(self.curse_add_line(msg, decoration='CPU_TIME', optional=True))
-                    msg = '{}:{}'.format(str(minutes).zfill(2), seconds)
-                else:
-                    msg = '{:>4}:{}.{}'.format(minutes, seconds, microseconds)
-        else:
-            msg = '{:>10}'.format('?')
-        ret.append(self.curse_add_line(msg, optional=True))
         # IO read/write
-        if 'io_counters' in p:
+        if 'io_counters' in p and p['io_counters'][4] == 1 and p['time_since_update'] != 0:
+            # Display rate if stats is available and io_tag ([4]) == 1
             # IO read
             io_rs = int((p['io_counters'][0] - p['io_counters'][2]) / p['time_since_update'])
             if io_rs == 0:
-                msg = '{:>6}'.format("0")
+                msg = self.layout_stat['ior'].format("0")
             else:
-                msg = '{:>6}'.format(self.auto_unit(io_rs, low_precision=True))
+                msg = self.layout_stat['ior'].format(self.auto_unit(io_rs,
+                                                                    low_precision=True))
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
             # IO write
             io_ws = int((p['io_counters'][1] - p['io_counters'][3]) / p['time_since_update'])
             if io_ws == 0:
-                msg = '{:>6}'.format("0")
+                msg = self.layout_stat['iow'].format("0")
             else:
-                msg = '{:>6}'.format(self.auto_unit(io_ws, low_precision=True))
+                msg = self.layout_stat['iow'].format(self.auto_unit(io_ws,
+                                                                    low_precision=True))
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
         else:
-            msg = '{:>6}'.format("?")
+            msg = self.layout_header['ior'].format("?")
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
+            msg = self.layout_header['iow'].format("?")
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
 
         # Command line
         # If no command line for the process is available, fallback to
         # the bare process name instead
-        cmdline = p['cmdline']
+        if 'cmdline' in p:
+            cmdline = p['cmdline']
+        else:
+            cmdline = '?'
         try:
-            # XXX: remove `cmdline != ['']` when we'll drop support for psutil<4.0.0
-            if cmdline and cmdline != ['']:
+            if cmdline:
                 path, cmd, arguments = split_cmdline(cmdline)
                 if os.path.isdir(path) and not args.process_short_name:
-                    msg = ' {}'.format(path) + os.sep
+                    msg = self.layout_stat['command'].format(path) + os.sep
                     ret.append(self.curse_add_line(msg, splittable=True))
-                    if glances_processes.is_tree_enabled():
-                        # mark position to add tree decoration
-                        ret[-1]["_tree_decoration"] = True
                     ret.append(self.curse_add_line(cmd, decoration='PROCESS', splittable=True))
                 else:
-                    msg = ' {}'.format(cmd)
+                    msg = self.layout_stat['command'].format(cmd)
                     ret.append(self.curse_add_line(msg, decoration='PROCESS', splittable=True))
-                    if glances_processes.is_tree_enabled():
-                        # mark position to add tree decoration
-                        ret[-1]["_tree_decoration"] = True
                 if arguments:
-                    msg = ' {}'.format(arguments)
+                    msg = ' ' + self.layout_stat['command'].format(arguments)
                     ret.append(self.curse_add_line(msg, splittable=True))
             else:
-                msg = ' {}'.format(p['name'])
+                msg = self.layout_stat['name'].format(p['name'])
                 ret.append(self.curse_add_line(msg, splittable=True))
         except UnicodeEncodeError:
             ret.append(self.curse_add_line('', splittable=True))
 
         # Add extended stats but only for the top processes
-        # !!! CPU consumption ???
-        # TODO: extended stats into the web interface
-        if first and 'extended_stats' in p:
+        if first and 'extended_stats' in p and args.enable_process_extended:
             # Left padding
             xpad = ' ' * 13
             # First line is CPU affinity
@@ -365,34 +340,33 @@ class Plugin(GlancesPlugin):
                 msg = xpad + 'CPU affinity: ' + str(len(p['cpu_affinity'])) + ' cores'
                 ret.append(self.curse_add_line(msg, splittable=True))
             # Second line is memory info
-            if 'memory_info' in p and p['memory_info'] is not None:
+            if 'memory_info' in p and \
+               p['memory_info'] is not None:
                 ret.append(self.curse_new_line())
-                msg = xpad + 'Memory info: '
-                for k, v in iteritems(p['memory_info']._asdict()):
-                    # Ignore rss and vms (already displayed)
-                    if k not in ['rss', 'vms'] and v is not None:
-                        msg += k + ' ' + self.auto_unit(v, low_precision=False) + ' '
+                msg = '{}Memory info: {}'.format(xpad, p['memory_info'])
                 if 'memory_swap' in p and p['memory_swap'] is not None:
-                    msg += 'swap ' + self.auto_unit(p['memory_swap'], low_precision=False)
+                    msg += ' swap ' + self.auto_unit(p['memory_swap'], low_precision=False)
                 ret.append(self.curse_add_line(msg, splittable=True))
             # Third line is for open files/network sessions
             msg = ''
             if 'num_threads' in p and p['num_threads'] is not None:
-                msg += 'threads ' + str(p['num_threads']) + ' '
+                msg += str(p['num_threads']) + ' threads '
             if 'num_fds' in p and p['num_fds'] is not None:
-                msg += 'files ' + str(p['num_fds']) + ' '
+                msg += str(p['num_fds']) + ' files '
             if 'num_handles' in p and p['num_handles'] is not None:
-                msg += 'handles ' + str(p['num_handles']) + ' '
+                msg += str(p['num_handles']) + ' handles '
             if 'tcp' in p and p['tcp'] is not None:
-                msg += 'TCP ' + str(p['tcp']) + ' '
+                msg += str(p['tcp']) + ' TCP '
             if 'udp' in p and p['udp'] is not None:
-                msg += 'UDP ' + str(p['udp']) + ' '
+                msg += str(p['udp']) + ' UDP'
             if msg != '':
                 ret.append(self.curse_new_line())
                 msg = xpad + 'Open: ' + msg
                 ret.append(self.curse_add_line(msg, splittable=True))
             # Fouth line is IO nice level (only Linux and Windows OS)
-            if 'ionice' in p and p['ionice'] is not None:
+            if 'ionice' in p and \
+               p['ionice'] is not None \
+               and hasattr(p['ionice'], 'ioclass'):
                 ret.append(self.curse_new_line())
                 msg = xpad + 'IO nice: '
                 k = 'Class is '
@@ -427,7 +401,7 @@ class Plugin(GlancesPlugin):
 
         return ret
 
-    def msg_curse(self, args=None):
+    def msg_curse(self, args=None, max_width=None):
         """Return the dict to display in the curse interface."""
         # Init the return message
         ret = []
@@ -443,67 +417,63 @@ class Plugin(GlancesPlugin):
         self.__msg_curse_header(ret, process_sort_key, args)
 
         # Process list
-        if glances_processes.is_tree_enabled():
-            ret.extend(self.get_process_tree_curses_data(
-                self.__sort_stats(process_sort_key), args, first_level=True,
-                max_node_count=glances_processes.max_processes))
-        else:
-            # Loop over processes (sorted by the sort key previously compute)
-            first = True
-            for p in self.__sort_stats(process_sort_key):
-                ret.extend(self.get_process_curses_data(p, first, args))
-                # End of extended stats
-                first = False
-            if glances_processes.process_filter is not None:
-                if args.reset_minmax_tag:
-                    args.reset_minmax_tag = not args.reset_minmax_tag
-                    self.__mmm_reset()
-                self.__msg_curse_sum(ret, args=args)
-                self.__msg_curse_sum(ret, mmm='min', args=args)
-                self.__msg_curse_sum(ret, mmm='max', args=args)
+        # Loop over processes (sorted by the sort key previously compute)
+        first = True
+        for p in self.__sort_stats(process_sort_key):
+            ret.extend(self.get_process_curses_data(p, first, args))
+            # End of extended stats
+            first = False
+        if glances_processes.process_filter is not None:
+            if args.reset_minmax_tag:
+                args.reset_minmax_tag = not args.reset_minmax_tag
+                self.__mmm_reset()
+            self.__msg_curse_sum(ret, args=args)
+            self.__msg_curse_sum(ret, mmm='min', args=args)
+            self.__msg_curse_sum(ret, mmm='max', args=args)
 
         # Return the message with decoration
         return ret
 
     def __msg_curse_header(self, ret, process_sort_key, args=None):
-        """
-        Build the header and add it to the ret dict
-        """
+        """Build the header and add it to the ret dict."""
         sort_style = 'SORT'
 
         if args.disable_irix and 0 < self.nb_log_core < 10:
-            msg = '{:>6}'.format('CPU%/' + str(self.nb_log_core))
+            msg = self.layout_header['cpu'].format('CPU%/' + str(self.nb_log_core))
         elif args.disable_irix and self.nb_log_core != 0:
-            msg = '{:>6}'.format('CPU%/C')
+            msg = self.layout_header['cpu'].format('CPU%/C')
         else:
-            msg = '{:>6}'.format('CPU%')
+            msg = self.layout_header['cpu'].format('CPU%')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'cpu_percent' else 'DEFAULT'))
-        msg = '{:>6}'.format('MEM%')
+        msg = self.layout_header['mem'].format('MEM%')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'memory_percent' else 'DEFAULT'))
-        msg = '{:>6}'.format('VIRT')
+        msg = self.layout_header['virt'].format('VIRT')
         ret.append(self.curse_add_line(msg, optional=True))
-        msg = '{:>6}'.format('RES')
+        msg = self.layout_header['res'].format('RES')
         ret.append(self.curse_add_line(msg, optional=True))
-        msg = '{:>{width}}'.format('PID', width=self.__max_pid_size() + 1)
+        msg = self.layout_header['pid'].format('PID', width=self.__max_pid_size())
         ret.append(self.curse_add_line(msg))
-        msg = ' {:10}'.format('USER')
+        msg = self.layout_header['user'].format('USER')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'username' else 'DEFAULT'))
-        msg = '{:>4}'.format('NI')
-        ret.append(self.curse_add_line(msg))
-        msg = '{:>2}'.format('S')
-        ret.append(self.curse_add_line(msg))
-        msg = '{:>10}'.format('TIME+')
+        msg = self.layout_header['time'].format('TIME+')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'cpu_times' else 'DEFAULT', optional=True))
-        msg = '{:>6}'.format('R/s')
+        msg = self.layout_header['thread'].format('THR')
+        ret.append(self.curse_add_line(msg))
+        msg = self.layout_header['nice'].format('NI')
+        ret.append(self.curse_add_line(msg))
+        msg = self.layout_header['status'].format('S')
+        ret.append(self.curse_add_line(msg))
+        msg = self.layout_header['ior'].format('R/s')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'io_counters' else 'DEFAULT', optional=True, additional=True))
-        msg = '{:>6}'.format('W/s')
+        msg = self.layout_header['iow'].format('W/s')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'io_counters' else 'DEFAULT', optional=True, additional=True))
-        msg = ' {:8}'.format('Command')
+        msg = self.layout_header['command'].format('Command')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'name' else 'DEFAULT'))
 
     def __msg_curse_sum(self, ret, sep_char='_', mmm=None, args=None):
         """
-        Build the sum message (only when filter is on) and add it to the ret dict
+        Build the sum message (only when filter is on) and add it to the ret dict.
+
         * ret: list of string where the message is added
         * sep_char: define the line separation char
         * mmm: display min, max, mean or current (if mmm=None)
@@ -514,67 +484,72 @@ class Plugin(GlancesPlugin):
             ret.append(self.curse_add_line(sep_char * 69))
             ret.append(self.curse_new_line())
         # CPU percent sum
-        msg = '{:>6.1f}'.format(self.__sum_stats('cpu_percent', mmm=mmm))
+        msg = self.layout_stat['cpu'].format(self.__sum_stats('cpu_percent', mmm=mmm))
         ret.append(self.curse_add_line(msg,
                                        decoration=self.__mmm_deco(mmm)))
         # MEM percent sum
-        msg = '{:>6.1f}'.format(self.__sum_stats('memory_percent', mmm=mmm))
+        msg = self.layout_stat['mem'].format(self.__sum_stats('memory_percent', mmm=mmm))
         ret.append(self.curse_add_line(msg,
                                        decoration=self.__mmm_deco(mmm)))
         # VIRT and RES memory sum
         if 'memory_info' in self.stats[0] and self.stats[0]['memory_info'] is not None and self.stats[0]['memory_info'] != '':
             # VMS
-            msg = '{:>6}'.format(self.auto_unit(self.__sum_stats('memory_info', indice=1, mmm=mmm), low_precision=False))
+            msg = self.layout_stat['virt'].format(self.auto_unit(self.__sum_stats('memory_info', indice=1, mmm=mmm), low_precision=False))
             ret.append(self.curse_add_line(msg,
                                            decoration=self.__mmm_deco(mmm),
                                            optional=True))
             # RSS
-            msg = '{:>6}'.format(self.auto_unit(self.__sum_stats('memory_info', indice=0, mmm=mmm), low_precision=False))
+            msg = self.layout_stat['res'].format(self.auto_unit(self.__sum_stats('memory_info', indice=0, mmm=mmm), low_precision=False))
             ret.append(self.curse_add_line(msg,
                                            decoration=self.__mmm_deco(mmm),
                                            optional=True))
         else:
-            msg = '{:>6}'.format('')
+            msg = self.layout_header['virt'].format('')
             ret.append(self.curse_add_line(msg))
+            msg = self.layout_header['res'].format('')
             ret.append(self.curse_add_line(msg))
         # PID
-        msg = '{:>6}'.format('')
+        msg = self.layout_header['pid'].format('', width=self.__max_pid_size())
         ret.append(self.curse_add_line(msg))
         # USER
-        msg = ' {:9}'.format('')
-        ret.append(self.curse_add_line(msg))
-        # NICE
-        msg = '{:>5}'.format('')
-        ret.append(self.curse_add_line(msg))
-        # STATUS
-        msg = '{:>2}'.format('')
+        msg = self.layout_header['user'].format('')
         ret.append(self.curse_add_line(msg))
         # TIME+
-        msg = '{:>10}'.format('')
+        msg = self.layout_header['time'].format('')
         ret.append(self.curse_add_line(msg, optional=True))
+        # THREAD
+        msg = self.layout_header['thread'].format('')
+        ret.append(self.curse_add_line(msg))
+        # NICE
+        msg = self.layout_header['nice'].format('')
+        ret.append(self.curse_add_line(msg))
+        # STATUS
+        msg = self.layout_header['status'].format('')
+        ret.append(self.curse_add_line(msg))
         # IO read/write
         if 'io_counters' in self.stats[0] and mmm is None:
             # IO read
             io_rs = int((self.__sum_stats('io_counters', 0) - self.__sum_stats('io_counters', indice=2, mmm=mmm)) / self.stats[0]['time_since_update'])
             if io_rs == 0:
-                msg = '{:>6}'.format('0')
+                msg = self.layout_stat['ior'].format('0')
             else:
-                msg = '{:>6}'.format(self.auto_unit(io_rs, low_precision=True))
+                msg = self.layout_stat['ior'].format(self.auto_unit(io_rs, low_precision=True))
             ret.append(self.curse_add_line(msg,
                                            decoration=self.__mmm_deco(mmm),
                                            optional=True, additional=True))
             # IO write
             io_ws = int((self.__sum_stats('io_counters', 1) - self.__sum_stats('io_counters', indice=3, mmm=mmm)) / self.stats[0]['time_since_update'])
             if io_ws == 0:
-                msg = '{:>6}'.format('0')
+                msg = self.layout_stat['iow'].format('0')
             else:
-                msg = '{:>6}'.format(self.auto_unit(io_ws, low_precision=True))
+                msg = self.layout_stat['iow'].format(self.auto_unit(io_ws, low_precision=True))
             ret.append(self.curse_add_line(msg,
                                            decoration=self.__mmm_deco(mmm),
                                            optional=True, additional=True))
         else:
-            msg = '{:>6}'.format('')
+            msg = self.layout_header['ior'].format('')
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
+            msg = self.layout_header['iow'].format('')
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
         if mmm is None:
             msg = ' < {}'.format('current')
@@ -586,30 +561,32 @@ class Plugin(GlancesPlugin):
             ret.append(self.curse_add_line(msg, optional=True))
 
     def __mmm_deco(self, mmm):
-        """
-        Return the decoration string for the current mmm status
-        """
+        """Return the decoration string for the current mmm status."""
         if mmm is not None:
             return 'DEFAULT'
         else:
             return 'FILTER'
 
     def __mmm_reset(self):
-        """
-        Reset the MMM stats
-        """
+        """Reset the MMM stats."""
         self.mmm_min = {}
         self.mmm_max = {}
 
     def __sum_stats(self, key, indice=None, mmm=None):
-        """
-        Return the sum of the stats value for the given key
+        """Return the sum of the stats value for the given key.
+
         * indice: If indice is set, get the p[key][indice]
         * mmm: display min, max, mean or current (if mmm=None)
         """
         # Compute stats summary
         ret = 0
         for p in self.stats:
+            if key not in p:
+                # Correct issue #1188
+                continue
+            if p[key] is None:
+                # Correct https://github.com/nicolargo/glances/issues/1105#issuecomment-363553788
+                continue
             if indice is None:
                 ret += p[key]
             else:
@@ -647,13 +624,12 @@ class Plugin(GlancesPlugin):
         return ret
 
     def __sort_stats(self, sortedby=None):
-        """Return the stats (dict) sorted by (sortedby)"""
+        """Return the stats (dict) sorted by (sortedby)."""
         return sort_stats(self.stats, sortedby,
-                          tree=glances_processes.is_tree_enabled(),
                           reverse=glances_processes.sort_reverse)
 
     def __max_pid_size(self):
-        """Return the maximum PID size in number of char"""
+        """Return the maximum PID size in number of char."""
         if self.pid_max is not None:
             return len(str(self.pid_max))
         else:
